@@ -34,7 +34,6 @@
 #include "presence.h"
 #include "presence-cache.h"
 #include "namespaces.h"
-#include "sha1/sha1.h"
 #include "vcard-manager.h"
 #include "util.h"
 
@@ -873,8 +872,7 @@ conn_avatars_init (GabbleConnection *conn)
 void
 conn_avatars_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcConnectionInterfaceAvatarsClass *klass =
-    (TpSvcConnectionInterfaceAvatarsClass *) g_iface;
+  TpSvcConnectionInterfaceAvatarsClass *klass = g_iface;
 
 #define IMPLEMENT(x) tp_svc_connection_interface_avatars_implement_##x (\
     klass, gabble_connection_##x)
