@@ -27,8 +27,9 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/channel-manager.h>
+#include <telepathy-glib/dbus.h>
 #include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/channel-factory-iface.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_IM
 
@@ -38,12 +39,11 @@
 #include "im-channel.h"
 #include "text-mixin.h"
 
-static void gabble_im_factory_iface_init (gpointer g_iface,
-    gpointer iface_data);
+static void channel_manager_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (GabbleImFactory, gabble_im_factory, G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
-      gabble_im_factory_iface_init));
+    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_MANAGER,
+      channel_manager_iface_init));
 
 /* properties */
 enum
@@ -58,6 +58,8 @@ struct _GabbleImFactoryPrivate
   LmMessageHandler *message_cb;
   GHashTable *channels;
 
+  gulong status_changed_id;
+
   gboolean dispose_has_run;
 };
 
@@ -81,21 +83,29 @@ gabble_im_factory_init (GabbleImFactory *self)
   self->priv->dispose_has_run = FALSE;
 }
 
+
+static void connection_status_changed_cb (GabbleConnection *conn,
+    guint status, guint reason, GabbleImFactory *self);
+
+
 static GObject *
 gabble_im_factory_constructor (GType type, guint n_props,
                                GObjectConstructParam *props)
 {
-  GObject *obj;
-  /* GabbleImFactoryPrivate *priv; */
-
-  obj = G_OBJECT_CLASS (gabble_im_factory_parent_class)->
+  GObject *obj = G_OBJECT_CLASS (gabble_im_factory_parent_class)->
            constructor (type, n_props, props);
-  /* priv = GABBLE_IM_FACTORY_GET_PRIVATE (obj); */
+  GabbleImFactory *self = GABBLE_IM_FACTORY (obj);
+
+  self->priv->status_changed_id = g_signal_connect (self->priv->conn,
+      "status-changed", (GCallback) connection_status_changed_cb, obj);
 
   return obj;
 }
 
 
+static void gabble_im_factory_close_all (GabbleImFactory *);
+
+
 static void
 gabble_im_factory_dispose (GObject *object)
 {
@@ -108,7 +118,7 @@ gabble_im_factory_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  gabble_im_factory_close_all (fac);
   g_assert (priv->channels == NULL);
 
   if (G_OBJECT_CLASS (gabble_im_factory_parent_class)->dispose)
@@ -169,19 +179,15 @@ gabble_im_factory_class_init (GabbleImFactoryClass *gabble_im_factory_class)
   object_class->set_property = gabble_im_factory_set_property;
 
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
-                                    "Gabble connection object that owns this "
-                                    "IM channel factory object.",
-                                    GABBLE_TYPE_CONNECTION,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+      "Gabble connection object that owns this IM channel factory object.",
+      GABBLE_TYPE_CONNECTION,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
 }
 
 static GabbleIMChannel *new_im_channel (GabbleImFactory *fac,
-    TpHandle handle, TpHandle initiator);
+    TpHandle handle, TpHandle initiator, gpointer request_token);
 
 static void im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data);
 
@@ -247,7 +253,7 @@ im_factory_message_cb (LmMessageHandler *handler,
 
       DEBUG ("found no IM channel, creating one");
 
-      chan = new_im_channel (fac, handle, handle);
+      chan = new_im_channel (fac, handle, handle, NULL);
     }
 
   g_assert (chan != NULL);
@@ -301,6 +307,14 @@ im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data)
 
   DEBUG ("%p, channel %p", self, chan);
 
+  g_object_get (chan,
+      "handle", &contact_handle,
+      "channel-destroyed", &really_destroyed,
+      NULL);
+
+  tp_channel_manager_emit_channel_closed_for_object (self,
+      (TpExportableChannel *) chan);
+
   if (priv->channels != NULL)
     {
       g_object_get (chan,
@@ -316,10 +330,11 @@ im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data)
         }
       else
         {
+
           DEBUG ("reopening channel with handle %u due to pending messages",
               contact_handle);
-          tp_channel_factory_iface_emit_new_channel (self,
-              (TpChannelIface *) chan, NULL);
+          tp_channel_manager_emit_new_channel (self,
+              (TpExportableChannel *) chan, NULL);
         }
     }
 }
@@ -330,12 +345,14 @@ im_channel_closed_cb (GabbleIMChannel *chan, gpointer user_data)
 static GabbleIMChannel *
 new_im_channel (GabbleImFactory *fac,
                 TpHandle handle,
-                TpHandle initiator)
+                TpHandle initiator,
+                gpointer request_token)
 {
   GabbleImFactoryPrivate *priv;
   TpBaseConnection *conn;
   GabbleIMChannel *chan;
   char *object_path;
+  GSList *request_tokens;
 
   g_return_val_if_fail (GABBLE_IS_IM_FACTORY (fac), NULL);
   g_return_val_if_fail (handle != 0, NULL);
@@ -356,155 +373,254 @@ new_im_channel (GabbleImFactory *fac,
 
   DEBUG ("object path %s", object_path);
 
+  g_free (object_path);
+
   g_signal_connect (chan, "closed", (GCallback) im_channel_closed_cb, fac);
 
   g_hash_table_insert (priv->channels, GINT_TO_POINTER (handle), chan);
 
-  tp_channel_factory_iface_emit_new_channel (fac, (TpChannelIface *) chan,
-      NULL);
+  if (request_token != NULL)
+    request_tokens = g_slist_prepend (NULL, request_token);
+  else
+    request_tokens = NULL;
 
-  g_free (object_path);
+  tp_channel_manager_emit_new_channel (fac,
+      (TpExportableChannel *) chan, request_tokens);
+
+  g_slist_free (request_tokens);
 
   return chan;
 }
 
 static void
-gabble_im_factory_iface_close_all (TpChannelFactoryIface *iface)
+gabble_im_factory_close_all (GabbleImFactory *self)
 {
-  GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-
-  DEBUG ("closing channels");
-
-  if (priv->channels)
+  if (self->priv->channels != NULL)
     {
-      GHashTable *tmp = priv->channels;
-      priv->channels = NULL;
+      GHashTable *tmp = self->priv->channels;
+
+      DEBUG ("closing channels");
+      self->priv->channels = NULL;
       g_hash_table_destroy (tmp);
     }
+
+  if (self->priv->status_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->priv->conn,
+          self->priv->status_changed_id);
+      self->priv->status_changed_id = 0;
+    }
+
+  if (self->priv->message_cb != NULL)
+    {
+      DEBUG ("removing callbacks");
+      lm_connection_unregister_message_handler (self->priv->conn->lmconn,
+          self->priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
+      lm_message_handler_unref (self->priv->message_cb);
+      self->priv->message_cb = NULL;
+    }
 }
 
-static void
-gabble_im_factory_iface_connecting (TpChannelFactoryIface *iface)
-{
-  GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-
-  DEBUG ("adding callbacks");
-
-  g_assert (priv->message_cb == NULL);
-
-  priv->message_cb = lm_message_handler_new (im_factory_message_cb, fac, NULL);
-  lm_connection_register_message_handler (priv->conn->lmconn, priv->message_cb,
-                                          LM_MESSAGE_TYPE_MESSAGE,
-                                          LM_HANDLER_PRIORITY_LAST);
-}
-
-
 
 static void
-gabble_im_factory_iface_connected (TpChannelFactoryIface *iface)
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabbleImFactory *self)
 {
-  /* nothing to do */
+  switch (status)
+    {
+    case TP_CONNECTION_STATUS_CONNECTING:
+      DEBUG ("adding callbacks");
+      g_assert (self->priv->message_cb == NULL);
+
+      self->priv->message_cb = lm_message_handler_new (im_factory_message_cb,
+          self, NULL);
+      lm_connection_register_message_handler (self->priv->conn->lmconn,
+          self->priv->message_cb, LM_MESSAGE_TYPE_MESSAGE,
+          LM_HANDLER_PRIORITY_LAST);
+      break;
+
+    case TP_CONNECTION_STATUS_DISCONNECTED:
+      gabble_im_factory_close_all (self);
+      break;
+    }
 }
 
-static void
-gabble_im_factory_iface_disconnected (TpChannelFactoryIface *iface)
-{
-  GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-
-  DEBUG ("removing callbacks");
-
-  g_assert (priv->message_cb != NULL);
-
-  lm_connection_unregister_message_handler (priv->conn->lmconn,
-      priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
-  lm_message_handler_unref (priv->message_cb);
-  priv->message_cb = NULL;
-}
 
 struct _ForeachData
 {
-  TpChannelFunc foreach;
+  TpExportableChannelFunc func;
   gpointer user_data;
 };
 
 static void
 _foreach_slave (gpointer key, gpointer value, gpointer user_data)
 {
-  struct _ForeachData *data = (struct _ForeachData *) user_data;
-  TpChannelIface *chan = TP_CHANNEL_IFACE (value);
+  struct _ForeachData *data = user_data;
+  TpExportableChannel *chan = TP_EXPORTABLE_CHANNEL (value);
 
-  data->foreach (chan, data->user_data);
+  data->func (chan, data->user_data);
 }
 
 static void
-gabble_im_factory_iface_foreach (TpChannelFactoryIface *iface,
-                                 TpChannelFunc foreach,
-                                 gpointer user_data)
+gabble_im_factory_foreach_channel (TpChannelManager *manager,
+                                   TpExportableChannelFunc func,
+                                   gpointer user_data)
 {
-  GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
+  GabbleImFactory *self = GABBLE_IM_FACTORY (manager);
   struct _ForeachData data;
 
   data.user_data = user_data;
-  data.foreach = foreach;
+  data.func = func;
 
-  g_hash_table_foreach (priv->channels, _foreach_slave, &data);
+  g_hash_table_foreach (self->priv->channels, _foreach_slave, &data);
 }
 
-static TpChannelFactoryRequestStatus
-gabble_im_factory_iface_request (TpChannelFactoryIface *iface,
-                                 const gchar *chan_type,
-                                 TpHandleType handle_type,
-                                 guint handle,
-                                 gpointer request,
-                                 TpChannelIface **ret,
-                                 GError **error)
-{
-  GabbleImFactory *fac = GABBLE_IM_FACTORY (iface);
-  GabbleImFactoryPrivate *priv = GABBLE_IM_FACTORY_GET_PRIVATE (fac);
-  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
-  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      base_conn, TP_HANDLE_TYPE_CONTACT);
-  GabbleIMChannel *chan;
-  TpChannelFactoryRequestStatus status;
 
-  if (strcmp (chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_IMPLEMENTED;
+static const gchar * const im_channel_fixed_properties[] = {
+    TP_IFACE_CHANNEL ".ChannelType",
+    TP_IFACE_CHANNEL ".TargetHandleType",
+    NULL
+};
 
-  if (handle_type != TP_HANDLE_TYPE_CONTACT)
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
+static const gchar * const im_channel_allowed_properties[] = {
+    TP_IFACE_CHANNEL ".TargetHandle",
+    TP_IFACE_CHANNEL ".TargetID",
+    NULL
+};
 
-  if (!tp_handle_is_valid (contact_repo, handle, error))
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_ERROR;
-
-  chan = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
-
-  status = TP_CHANNEL_FACTORY_REQUEST_STATUS_EXISTING;
-  if (!chan)
-    {
-      status = TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED;
-      chan = new_im_channel (fac, handle, base_conn->self_handle);
-    }
-
-  g_assert (chan);
-  *ret = TP_CHANNEL_IFACE (chan);
-  return status;
-}
 
 static void
-gabble_im_factory_iface_init (gpointer g_iface,
-                              gpointer iface_data)
+gabble_im_factory_foreach_channel_class (TpChannelManager *manager,
+    TpChannelManagerChannelClassFunc func,
+    gpointer user_data)
 {
-  TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
+  GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+  GValue *value;
 
-  klass->close_all = gabble_im_factory_iface_close_all;
-  klass->connecting = gabble_im_factory_iface_connecting;
-  klass->connected = gabble_im_factory_iface_connected;
-  klass->disconnected = gabble_im_factory_iface_disconnected;
-  klass->foreach = gabble_im_factory_iface_foreach;
-  klass->request = gabble_im_factory_iface_request;
+  value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (value, TP_IFACE_CHANNEL_TYPE_TEXT);
+  g_hash_table_insert (table, (gchar *) im_channel_fixed_properties[0],
+      value);
+
+  value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (table, (gchar *) im_channel_fixed_properties[1],
+      value);
+
+  func (manager, table, im_channel_allowed_properties, user_data);
+
+  g_hash_table_destroy (table);
 }
 
+
+static gboolean
+gabble_im_factory_requestotron (GabbleImFactory *self,
+                                gpointer request_token,
+                                GHashTable *request_properties,
+                                gboolean require_new)
+{
+  TpBaseConnection *base_conn = (TpBaseConnection *) self->priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      base_conn, TP_HANDLE_TYPE_CONTACT);
+  TpHandle handle;
+  GError *error = NULL;
+  TpExportableChannel *channel;
+
+  if (tp_strdiff (tp_asv_get_string (request_properties,
+          TP_IFACE_CHANNEL ".ChannelType"), TP_IFACE_CHANNEL_TYPE_TEXT))
+    return FALSE;
+
+  if (tp_asv_get_uint32 (request_properties,
+        TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_CONTACT)
+    return FALSE;
+
+  handle = tp_asv_get_uint32 (request_properties,
+      TP_IFACE_CHANNEL ".TargetHandle", NULL);
+
+  if (!tp_handle_is_valid (contact_repo, handle, &error))
+    goto error;
+
+  if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+          im_channel_fixed_properties, im_channel_allowed_properties,
+          &error))
+    goto error;
+
+  channel = g_hash_table_lookup (self->priv->channels,
+      GUINT_TO_POINTER (handle));
+
+  if (channel == NULL)
+    {
+      new_im_channel (self, handle, base_conn->self_handle, request_token);
+      return TRUE;
+    }
+
+  if (require_new)
+    {
+      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Already chatting with contact #%u in another channel", handle);
+      goto error;
+    }
+
+  tp_channel_manager_emit_request_already_satisfied (self, request_token,
+      channel);
+  return TRUE;
+
+error:
+  tp_channel_manager_emit_request_failed (self, request_token,
+      error->domain, error->code, error->message);
+  g_error_free (error);
+  return TRUE;
+}
+
+
+static gboolean
+gabble_im_factory_create_channel (TpChannelManager *manager,
+                                  gpointer request_token,
+                                  GHashTable *request_properties)
+{
+  GabbleImFactory *self = GABBLE_IM_FACTORY (manager);
+
+  return gabble_im_factory_requestotron (self, request_token,
+      request_properties, TRUE);
+}
+
+
+static gboolean
+gabble_im_factory_request_channel (TpChannelManager *manager,
+                                   gpointer request_token,
+                                   GHashTable *request_properties)
+{
+  GabbleImFactory *self = GABBLE_IM_FACTORY (manager);
+
+  return gabble_im_factory_requestotron (self, request_token,
+      request_properties, FALSE);
+}
+
+
+static gboolean
+gabble_im_factory_ensure_channel (TpChannelManager *manager,
+                                  gpointer request_token,
+                                  GHashTable *request_properties)
+{
+  GabbleImFactory *self = GABBLE_IM_FACTORY (manager);
+
+  return gabble_im_factory_requestotron (self, request_token,
+      request_properties, FALSE);
+}
+
+
+static void
+channel_manager_iface_init (gpointer g_iface,
+                            gpointer iface_data)
+{
+  TpChannelManagerIface *iface = g_iface;
+
+  iface->foreach_channel = gabble_im_factory_foreach_channel;
+  iface->foreach_channel_class = gabble_im_factory_foreach_channel_class;
+  iface->create_channel = gabble_im_factory_create_channel;
+  iface->request_channel = gabble_im_factory_request_channel;
+  iface->ensure_channel = gabble_im_factory_ensure_channel;
+}
