@@ -92,6 +92,9 @@ LmMessage *gabble_presence_as_message (GabblePresence *presence,
     const gchar *to);
 gchar *gabble_presence_dump (GabblePresence *presence);
 
+gboolean gabble_presence_added_to_view (GabblePresence *presence);
+gboolean gabble_presence_removed_from_view (GabblePresence *presence);
+
 G_END_DECLS
 
 #endif /* __GABBLE_PRESENCE_H__ */
