@@ -17,11 +17,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "tube-iface.h"
-#include "gabble-connection.h"
-#include "extensions/extensions.h"
 
-#include <glib.h>
+#include "connection.h"
 
 gboolean
 gabble_tube_iface_accept (GabbleTubeIface *self,
