@@ -17,11 +17,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
 #include "tube-stream.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -30,25 +29,23 @@
 #include <errno.h>
 #include <netdb.h>
 
-#include <glib.h>
 #include <glib/gstdio.h>
-
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/gtypes.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_TUBES
 
-#include "debug.h"
-#include "disco.h"
-#include "extensions/extensions.h"
-#include "gabble-connection.h"
-#include "presence.h"
-#include "presence-cache.h"
-#include "namespaces.h"
-#include "util.h"
-#include "tube-iface.h"
 #include "bytestream-factory.h"
 #include "bytestream-iface.h"
+#include "connection.h"
+#include "debug.h"
+#include "disco.h"
 #include "gabble-signals-marshal.h"
+#include "namespaces.h"
+#include "presence-cache.h"
+#include "presence.h"
+#include "tube-iface.h"
+#include "util.h"
 
 static void
 tube_iface_init (gpointer g_iface, gpointer iface_data);
@@ -56,14 +53,6 @@ tube_iface_init (gpointer g_iface, gpointer iface_data);
 G_DEFINE_TYPE_WITH_CODE (GabbleTubeStream, gabble_tube_stream, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_TUBE_IFACE, tube_iface_init));
 
-#define SOCKET_ADDRESS_IPV4_TYPE \
-    dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING, G_TYPE_UINT, \
-        G_TYPE_INVALID)
-
-#define SOCKET_ADDRESS_IPV6_TYPE \
-    dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING, G_TYPE_UINT, \
-        G_TYPE_INVALID)
-
 /* Linux glibc bits/socket.h suggests that struct sockaddr_storage is
  * not guaranteed to be big enough for AF_UNIX addresses */
 typedef union
@@ -105,7 +94,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleTubeStreamPrivate GabbleTubeStreamPrivate;
 struct _GabbleTubeStreamPrivate
 {
   GabbleConnection *conn;
@@ -132,8 +120,7 @@ struct _GabbleTubeStreamPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_TUBE_STREAM_GET_PRIVATE(obj) \
-    ((GabbleTubeStreamPrivate *) obj->priv)
+#define GABBLE_TUBE_STREAM_GET_PRIVATE(obj) ((obj)->priv)
 
 static void data_received_cb (GabbleBytestreamIface *ibb, TpHandle sender,
     GString *data, gpointer user_data);
@@ -325,7 +312,7 @@ start_stream_initiation (GabbleTubeStream *self,
   priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
 
   contact_repo = tp_base_connection_get_handles (
-     (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_CONTACT);
+     (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   jid = tp_handle_inspect (contact_repo, priv->initiator);
 
@@ -499,7 +486,7 @@ new_connection_to_socket (GabbleTubeStream *self,
 
       addr.un.sun_family = PF_UNIX;
       strncpy (addr.un.sun_path, array->data, sizeof (addr.un.sun_path) - 1);
-      addr.un.sun_path[sizeof (addr.un.sun_path)] = '\0';
+      addr.un.sun_path[sizeof (addr.un.sun_path) - 1] = '\0';
       len = sizeof (addr.un);
 
       DEBUG ("Will try to connect to socket: %s", (const gchar *) array->data);
@@ -689,9 +676,11 @@ tube_stream_open (GabbleTubeStream *self,
 
       DEBUG ("create socket %s:%u", "127.0.0.1", ntohs (addr.sin_port));
 
-      priv->address = tp_g_value_slice_new (SOCKET_ADDRESS_IPV4_TYPE);
+      priv->address = tp_g_value_slice_new (
+          TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4);
       g_value_take_boxed (priv->address,
-          dbus_g_type_specialized_construct (SOCKET_ADDRESS_IPV4_TYPE));
+          dbus_g_type_specialized_construct (
+              TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4));
 
       dbus_g_type_struct_set (priv->address,
           0, "127.0.0.1",
@@ -737,9 +726,11 @@ tube_stream_open (GabbleTubeStream *self,
 
       DEBUG ("create socket %s:%u", "::1", ntohs (addr.sin6_port));
 
-      priv->address = tp_g_value_slice_new (SOCKET_ADDRESS_IPV6_TYPE);
+      priv->address = tp_g_value_slice_new (
+          TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV6);
       g_value_take_boxed (priv->address,
-          dbus_g_type_specialized_construct (SOCKET_ADDRESS_IPV6_TYPE));
+          dbus_g_type_specialized_construct (
+              TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV6));
 
       dbus_g_type_struct_set (priv->address,
           0, "::1",
@@ -1330,7 +1321,7 @@ gabble_tube_stream_close (GabbleTubeIface *tube)
       LmMessage *msg;
       const gchar *jid;
       TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-          (TpBaseConnection*) priv->conn, TP_HANDLE_TYPE_CONTACT);
+          (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
       gchar *id_str;
 
       jid = tp_handle_inspect (contact_repo, priv->handle);
@@ -1517,7 +1508,7 @@ check_ip_params (TpSocketAddressType address_type,
   /* Check address type */
   if (address_type == TP_SOCKET_ADDRESS_TYPE_IPV4)
     {
-      if (G_VALUE_TYPE (address) != SOCKET_ADDRESS_IPV4_TYPE)
+      if (G_VALUE_TYPE (address) != TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4)
         {
           g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
               "IPv4 socket address is supposed to be sq");
@@ -1526,7 +1517,7 @@ check_ip_params (TpSocketAddressType address_type,
     }
   else if (address_type == TP_SOCKET_ADDRESS_TYPE_IPV6)
     {
-      if (G_VALUE_TYPE (address) != SOCKET_ADDRESS_IPV6_TYPE)
+      if (G_VALUE_TYPE (address) != TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV6)
         {
           g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
               "IPv6 socket address is supposed to be sq");
