@@ -1,7 +1,9 @@
 
+#include "config.h"
+
 #include <string.h>
 
-#include "util.h"
+#include "src/util.h"
 
 static gboolean
 test1 (void)
