@@ -57,14 +57,9 @@ def test(q, bus, conn, stream):
             'org.freedesktop.Telepathy.Channel',
             dbus_interface='org.freedesktop.DBus.Properties')
     assert channel_props['TargetID'] == 'chat@conf.localhost', channel_props
-
-    # Exercise FUTURE properties
-    future_props = text_chan.GetAll(
-            'org.freedesktop.Telepathy.Channel.FUTURE',
-            dbus_interface='org.freedesktop.DBus.Properties')
-    assert future_props['Requested'] == False
-    assert future_props['InitiatorID'] == 'bob@localhost'
-    assert future_props['InitiatorHandle'] == bob_handle
+    assert channel_props['Requested'] == False
+    assert channel_props['InitiatorID'] == 'bob@localhost'
+    assert channel_props['InitiatorHandle'] == bob_handle
 
     # accept the invitation
     call_async(q, group_iface, 'AddMembers', [room_self_handle], 'Oh, OK then')
